/*
 * freetrans - freetrans.h
 * 	version 0.0.0
 *
 * Copyright (C) 2005, 2006  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef FREETRANS_H_INCLUDED
#define FREETRANS_H_INCLUDED

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>

#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#elif HAVE_STDINT_H
#include <stdint.h>
#else
typedef signed char int8_t;
typedef signed short int16_t;
typedef signed int int32_t;

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;
#endif

#if __GNUC__ > 2
#define likely(x)	__builtin_expect(!!(x), 1)
#define unlikely(x)	__builtin_expect(!!(x), 0)
#define __FUNC		__PRETTY_FUNCTION__

#else
#define likely(x)	(x)
#define unlikely(x)	(x)
#define __FUNC		"unknown"

#warning "Please update me for your compiler."
#endif

#define fatal(fmt...)							\
	do {								\
		fprintf(stderr, fmt);					\
		abort();						\
	} while (0)

#define BUG()								\
	do {								\
		fprintf(stderr, "BUG in %s at %s:%i.", __FUNC,		\
				__FILE__, __LINE__);			\
		abort();						\
	} while (0)

#define BUG_ON(cond)							\
	do {								\
		if (unlikely(cond))					\
			BUG();						\
	} while (0)

#ifdef NDEBUG
#define BUG_ON_D(cond)	do { } while (0)
#else
#define BUG_ON_D(cond)							\
	do {								\
		if (unlikely(cond))					\
			BUG();						\
	} while (0)
#endif

#ifdef HAVE_ENDIAN_H
#include <endian.h>
#endif

#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#define my_bswap_16(x) bswap_16(x)
#define my_bswap_32(x) bswap_32(x)
#elif HAVE_SYS_ENDIAN_H
#include <sys/endian.h>
#define my_bswap_16(x) bswap16(x)
#define my_bswap_32(x) bswap32(x)
#else

static inline uint16_t my_bswap_16(uint16_t x)
{
	return ((x >> 8) | (x << 8));
}

static inline uint32_t my_bswap_32(uint32_t x)
{
	return ((x >> 24) | ((x >> 8) & 0xff00) | ((x << 8) & 0xff0000)
			| ((x << 24) & 0xff000000));
}

#endif

#if BYTE_ORDER == BIG_ENDIAN

#define cpu_to_be16(x)		(x)
#define cpu_to_be32(x)		(x)
#define cpu_to_le16(x)		(my_bswap_16(x))
#define cpu_to_le32(x)		(my_bswap_32(x))

#define be16_to_cpu(x)		(x)
#define be32_to_cpu(x)		(x)
#define le16_to_cpu(x)		(my_bswap_16(x))
#define le32_to_cpu(x)		(my_bswap_32(x))

#elif BYTE_ORDER == LITTLE_ENDIAN

#define cpu_to_be16(x)		(my_bswap_16(x))
#define cpu_to_be32(x)		(my_bswap_32(x))
#define cpu_to_le16(x)		(x)
#define cpu_to_le32(x)		(x)

#define be16_to_cpu(x)		(my_bswap_16(x))
#define be32_to_cpu(x)		(my_bswap_32(x))
#define le16_to_cpu(x)		(x)
#define le32_to_cpu(x)		(x)
#else
#error "Only little and big endian systems are supported."
#endif

#endif
